@@ -66,7 +66,7 @@ smooth_cifti <- function(
     res <- infer_resolution(x)
     if (any(res %in% c(NA, NaN))) {
       stop("Cannot smooth because the resolution could not be inferred.")
-    } else if (res[1] != res[2]) {
+    } else if (!(any(res == 0)) && res[1] != res[2]) {
       stop("Cannot smooth because the left and right cortex have unequal inferred resolutions: ", res[1], " and ", res[2], ".")
     }
 
